﻿<?php
require_once('register.php');

class Manager{// extends Register{{
	
	// Instance unique du manager pour le singleton
	private static $_instance;
	// Controller de la page en cours
	private static $_controller;
	// Action de la page en cours
	private static $_action;
	// Tableau de configuration pour les données
	private static $_conf;
	// Niveau du profile pour les droits d'accès
	private static $_profile;
	// Chemin vers le fichier de configuration
	private static $_path;

	/* Constructeur de la vue prenant en paramètres le controller et l'action*/
	protected function __construct($controllerClassName, $action){
		// parent::getInstance($controllerClassName, $action);
		// Définition des constantes  ROOT et WEBROOT pour les chemins

		define('WEBROOT', str_replace('index.php', '', $_SERVER['SCRIPT_NAME']));
		define('ROOT', str_replace('index.php', '', $_SERVER['SCRIPT_FILENAME']));

		// Construction des paramètres de la classe
		self::setPath("core/conf.ini");
		self::setAction($action);
		self::setController($controllerClassName);
		// Récupération des paramètres du fichier
		self::setConf();
		

	}

	// Récupération de l'instance du singleton
	public static function getInstance($conf){
		if(!is_null(self::$_instance)) {

			self::setAction($conf['action']);
			self::setController($conf['controller']);

			return self::$_instance;
		}
		else
			return self::$_instance = new Manager($conf['controller'], $conf['action']);
	}

	/* Setter de l'action pour la redéfinir à la volée*/
	public function setAction($action){
		self::$_conf['action'] = $action;
	}

	/* Getter de l'action pour tester l'action en cours*/
	public function getAction(){
		return self::$_conf['action'];
	}

	/* Setter du contoller pour le redéfinir à la volée*/
	public function setController($controllerClassName){
		self::$_conf['controller'] = $controllerClassName;
	}

	/* Getter du controller pour tester le controller en cours*/
	public function getController(){
		return self::$_conf['controller'];
	}

	/* Setter du chemin du fichier de configuration*/
	public function setPath($path){
		self::$_conf['path'] = $path;
	}

	/* Getter du chemin du fichier de configuration*/
	public function getPath(){
		return self::$_conf['path'];
	}

	// Récupération du contenu du fichier de configuration
	private function setConf(){
		$conf = file(self::getPath(), FILE_IGNORE_NEW_LINES|FILE_SKIP_EMPTY_LINES);
		if($conf){
			foreach($conf as $line){
				$param = explode("=",$line);
				self::$_conf[$param[0]]=$param[1];
			}
		}
	}
	
	public static function getConf($index){
		return self::$_conf[$index];
	}
	
	// Chargement de la page
	public function chargePage(){
		
		// Chargement des classes mères
		require_once('model.php');
		require_once('controller.php');
		require_once('view.php');
		
		// Récupération de l'action qui sera appelée pour chaque partie du MVC
		$action = self::getAction();
		$controller = self::getController();
		
		// Création du registre de données
		// $register = new Register($controller, $action);
		
		// Test de l'existance des fichiers à charger
		if(file_exists('models/'.$controller.'.php')){
			require('models/'.$controller.'.php');
			$model = ucfirst($controller)."_model";
			$obj_model = new $model($controller, $action);
		}

		$controllerClassPath = 'controllers/'.$controller.'.php';
		$viewClassPath = 'views/'.$controller.'/'.$controller.'.php';

		try{
			
			if(!file_exists($controllerClassPath))
				throw new Exception("Méthode du controller inaccessible");

			require($controllerClassPath);
			$controllerClassName = ucfirst($controller)."_controller";
			$obj_controller = new $controllerClassName($controller, $action);

			if(!method_exists($obj_controller, $action))
				throw new Exception("Méthode du controller inaccessible");

			$obj_controller->$action();
		
			if(!file_exists($viewClassPath))
				throw new Exception("Méthode du view inaccessible");

			require($viewClassPath);
			$viewClassName = ucfirst($controller)."_view";
			$obj_view = new $viewClassName($controller, $action);
			// var_dump($obj_view);
		
			if(!method_exists($obj_view, $action))
				throw new Exception("Méthode du view inaccessible");

			$obj_view->$action();
			
		// Si l'une des verifications ne passe pas, on charge la page d'erreur
		} catch (Exception $e){
			
			self::setController("error");
			self::setAction("error404");
			
			$action = self::getAction();
			$controller = self::getController();

			$controllerClassPath = 'controllers/'.$controller.'.php';
			$viewClassPath = 'views/'.$controller.'/'.$controller.'.php';
			
			require($controllerClassPath);
			$controllerClassName = ucfirst($controller)."_controller";
			$obj_controller = new $controllerClassName($controller, $action);
			$obj_controller->$action();
		
			require($viewClassPath);
			$viewClassName = ucfirst($controller)."_view";
			$obj_view = new $viewClassName($controller, $action);
			$obj_view->$action();
			
		}
		
		// Test de l'existance d'un model lié à la page puis chargement des données
		// Chargement des donnes optionnel, toutes les pages ne nécessitent pas d'accès à un base
		// donc autant économiser l'opération de création/récupération de la connexion
		if(isset($obj_model) && method_exists($obj_model, $action)){
			// Chargement
			$obj_model->$action();
			if($obj_model->validateModel()){
				echo "je charge les données";
				$obj_controller::setPageDatas($obj_model::getArrayList());
			} else {
				echo "Je ne charge pas les données, et peut-etre même que je fais une erreur";
			}
		}

		// AFFICHAGE DE LA PAGE AVEC LES ELEMENTS CHARGES ET CORRECT OU REDIRIGES VERS LA PAGE D'ERREUR

		// HEADER : Chargement du layout défini
		require('views/layout/headers/'.$obj_view->getLayout().'.php');

		// BODY : Affichage de la vue associée à l'action
		try{
			$obj_view->render($controller, $action);
		} catch(Exception $e) {
			$obj_view->render("error","error404");
		}

		// FOOTER : Chargement du layout défini
		require('views/layout/footers/'.$obj_view->getLayout($controllerClassName).'.php');
		
	}
	
}
?>